/**
 * 事务页面。 根据一站式服务大厅的事务接口。达到增删发起事务流程
 * 2019年10月23日 09:48:13
 * 钟是志
 *
 * */

import { apiRequest } from '../request';
import { getFormArrayConfig }  from '../config/index';
import { setHuanGeToken } from '@/utils/authority';
import { isJSON } from '@/baseComponent/utils';
import config from '@/config/config';

const giveValue = (x) => {
  if(x && x.rows){
    let res = [];
    for(let item of x.rows){
      res.push({
        ...item.hisTaskListData,
        ...item,
      });
    }
    x.rows = res;
    return x;
  }else{
    return {
      rows: [],
      total: 1,
    };
  }
};

const getOtherToken = () => {
    let userName =  sessionStorage.getItem("currentUser");
    if(isJSON(userName)){
      let name = JSON.parse(userName).name;
      return apiRequest(`/UserApi/loginByCount?username=${name}`, {}).then((res)=>{
        if(res){
          setHuanGeToken(res.token);
        }
      });
    }
};

const getId = (pathname) => { // 获取流程引擎 事务workId 和表dataBaseId
  let idObj = config.twOneStopConfig[pathname]; // 路由 /tw/Pjpy/YXTYSBB/Apply
  if (typeof idObj === 'undefined') {
    console.error('没有找到对应的流程引擎id');
    return false;
  } else {
    return {
      dataBaseId: idObj.id,
      workId: idObj.id,
    };
  }
};

const getColumns2 = (id) => { // 获取表头 workId
  return apiRequest('/DataColumnApi/getHeaderList', { dataObjId: id }).then((response)=>{
    if(response){
      return response.map((x)=>{
        return {
          title: x.title,
          dataIndex: x.name,
          type: x.type,
        }
      })
    }
  });
};

const getPages = (info) => { // 获取代办事项 分页数据 workId
  // workId, pageSize, pageNo,
  if (typeof info.pageNo === 'undefined') {
    info.pageNo = 1;
    info.pageSize = 10;
  }
  return apiRequest('/UnifiedAppFormApi/getWaitPage', info);
};

const getQueryCondition = (dataObjId) => { // 获取搜索条件  dataBaseId //暂时未生效. 因为欢哥那边不支持
  return [];
  return apiRequest('/DataColumnApi/getHeaderList', { dataObjId }).then((response) => {
    let res = [];
    if(response){
      for(let item of response){
        if(item.isShowQuery){
          let sear = getFormArrayConfig([item]);
          if(!Array.isArray(sear)){
            return  false;
          }
          delete sear[0].required;
          delete sear[0].placeHolder;
          res.push(sear[0]);
        }
      }
    }
    return res;
  });
};

const getListPage = (info) => { // 获取表数据。dataObjId
  if (typeof info.pageNo === 'undefined') {
    info.pageNo = 1;
    info.pageSize = 10;
  }
  return apiRequest('/DataObjApi/getFormDataPage', info);
};

const deleteFormData = (dataBaseId, id) => { // 删除表数据.
  const info = {
    keys: JSON.stringify({id}),
    objId: dataBaseId,
  };
  return apiRequest('/DataObjApi/deleteFormData', info);
};

/****************************** 以上接口暂时不用 ****************************/




const getColumns = (workId) => { // 获取表头 workId
  return apiRequest('/UnifiedAppFormApi/getFormTitle', { id: workId });
};


/**
 * 获取下拉枚举值
 * */
const getOptions  = async (id, filterSql = undefined, key, name) => {
  return await apiRequest('/DataColumnApi/getOptions', {id, filterSql}).then((x)=>{
    if(x){
      return x.map((y)=>{
        return {
          key: y[key],
          name: y[name],
        }
      })
    }else{
      return [];
    }
  });
};

/**
 * 获取下拉枚举值 通过sql语句
 * */
const getSqlOptions  = async (id, sqlKey = undefined, key, name ) => {
  return await apiRequest('/DataColumnApi/getSqlOptions', {id, sqlKey}).then((x)=>{
    if(x){
      return x.map((y)=>{
        return {
          key: y[key],
          name: y[name],
        }
      })
    }else{
      return [];
    }
  });
};

/**
 * 获取发起流程填写的字段表单详情
 * */
const getFormDetail = (id) => {
  return apiRequest('/UnifiedAppApi/getDetail', {id}).then((x)=>{
    if(x){
      return x;
    }else{
      return {};
    }
  });
};


/**
 *  发起流程
 * */
const startProcess = (info) => {
  return apiRequest('/UnifiedAppFormApi/startProcess', info).then((x)=>{
    if(x){
      return x;
    }else{
      return null;
    }
  });
};

/**
 *  查询已发起的流程的分页接口
 *
 * */

const getApplyPage = (info) => {
  return apiRequest('/UnifiedAppFormApi/getApplyPage', info).then((x)=>{
    return giveValue(x);

  });
};
/**
 *  查询一个流程的详情数据
 *
 * */

const getDetailInfo = (id) => {
  return apiRequest('/UnifiedAppFormApi/getFormDetail', {id}).then((x)=>{
      if(x){
        return x;
      }else{
        return null;
      }
  });
  };
/**
 * 待办业务
 * pageSize: 10
 * appId: 1185524814832467968
 * pageNo: 1
 * */
const getWaitPage = (info) => {
    return apiRequest('/UnifiedAppFormApi/getWaitPage', info).then((x) => {
       return giveValue(x);
    })
};
/**
 * 已办业务
 * pageSize: 10
 * appId: 1185524814832467968
 * pageNo: 1
 * */
const getHandledPage = (info) => {
  return apiRequest('/UnifiedAppFormApi/getHandledPage', info).then((x) => {

    return giveValue(x);
  })
};

/**
 * 审核接口,支持批量
 *  taskIds: 08d8d359-f7a6-11e9-9a27-0242b1fd760b
 *  examineMap: {"examine":"0","reason":"通过"}
 *  taskForm: {}
 * */
const handleAudit = (info) => {
  return apiRequest('/UnifiedAppFormApi/examineProcess', info).then((x) => {
    if(x){
      return x;
    }
  })
};

const handleSqlData = (info) => {
  return apiRequest('/DataColumnApi/getSqlData', info).then((x) => {
    if(x){
      return x;
    }else{
      return [];
    }
  })
};

const handleSqlfind = async(sqlKey = '') => {
  return await apiRequest('/SqlManageEntityApi/find', {sqlKey}).then((x) => {
    if(x){
      return x;
    }else{
      return null;
    }
  })
};

/**
 * 流程紧急度
 * 2019年12月5日
 * 钟是志
 * 先默认给后台传个值 保证能提交申请
 * */
const getSeriousOptions = () => {
  return apiRequest('/UnifiedAppFormLevelApi/getAll', {}).then((x)=>{
    if(x){
      return x.map((s)=>{
        return {
          key: s.id,
          name: s.name,
          color: s.color,
        }
      })
    }
  });

};

export {
  getId, getColumns, getPages, getQueryCondition,
  getListPage, deleteFormData, getColumns2, getFormDetail,
  getOptions, startProcess, getApplyPage, getOtherToken,
  getDetailInfo, getWaitPage, getHandledPage, handleAudit,
  handleSqlData,handleSqlfind, getSqlOptions, getSeriousOptions,
};


