import React, { Fragment } from 'react';
import { Divider, message, Modal, Popconfirm } from 'antd';
import moment from 'moment';
import router from 'umi/router';
// import ExportInfo from '../Export/index';
import { text, number, date, getFormArrayConfig } from '../config/index';
import ButtonDiy from '@/baseComponent/ButtonDiy';
import Shell from '../Shell';
import FormArray from '../AffairPage/component/FormArray';
import Edit from '../Edit/index';
// import ImportUtil from '../ImportUtil/ImportUtil';
import SearchDom from '@/highOrderComponent/SearchDom';
import StandardTable from '@/components/StandardTable';
import { getHead, fetchData, deleteItem, add } from './services';

const getValue = obj =>
  Object.keys(obj)
    .map(key => obj[key])
    .join(',');

const CreateForm = props => {
  const {
    modalVisible,
    handleAdd,
    formData,
    isAdd,
    handleModalVisible,
    formItem,
    changeFormData,
  } = props;

  // 通过回调
  const okHandle = () => {
    handleAdd({}, isAdd);
  };
  // 需要获取动态表单
  return (
    <Modal
      width="1000px"
      maskClosable={false}
      destroyOnClose
      title={isAdd ? '新增' : '编辑'}
      visible={modalVisible}
      onOk={okHandle}
      onCancel={() => handleModalVisible()}
    >
      <FormArray
        config={getFormArrayConfig(formItem)}
        value={formData}
        fileSpan={{ big: 2, small: 2 }}
        changeValue={changeFormData}
      />
      <div style={{ clear: 'both' }} />
    </Modal>
  );
};

class FormList extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      modalVisible: false,
      formData: null,
      formItem: [],
      searchConfig: [],
      formValues: {},
      isAdd: false,
      selectedRows: [],
      primaryKey: null,
      showEdit: false,
      pageData: {
        list: [],
        pagination: {},
      },
    };
    this.columns = [];
  }

  // 渲染值
  componentDidMount() {
    const { objId } = this.props;
    getHead({ dataObjId: objId }, datas => {
      if (!datas) return;
      this.setState({ formItem: datas });
      let i = 0;
      for (let t in datas) {
        if (i < 10) {
          let column = {};
          column.title = datas[t].title;
          column.dataIndex = datas[t].name;
          if (date.includes(datas[t].type)) {
            column.render = val => val && moment(val).format('YYYY-MM-DD HH:mm:ss');
          }
          this.columns.push(column);
          i++;
        }
        if (datas[t].isPrimaryKey) {
          this.setState({
            primaryKey: datas[t].name,
          });
        }
      }
      let searchConfig = [];
      for (let item of datas) {
        if (item.isShowQuery) {
          let sear = getFormArrayConfig([item]);
          if (!Array.isArray(sear)) {
            return false;
          }
          delete sear[0].required;
          delete sear[0].placeHolder;
          searchConfig.push(sear[0]);
        }
      }
      this.setState({
        searchConfig,
      });
      let opt = {
        title: '操作',
        render: (text, record) => (
          <Fragment>
            <div>
              <a onClick={this.modify.bind(this, record)}>编辑</a>
              <Divider type="vertical" />
              <Popconfirm
                title="确定删除该数据?"
                onConfirm={this.delete.bind(this, record)}
                okText="确定"
                cancelText="取消"
              >
                <a>删除</a>
              </Popconfirm>
            </div>
          </Fragment>
        ),
      };
      this.columns.push(opt);
      this.getPage();
    });
  }

  modify = record => {
    const { formItem } = this.state;
    if (formItem.length < 10) {
      this.setState({
        formData: { ...record },
        modalVisible: true,
        isAdd: false,
      });
    } else {
      this.setState({
        formData: { ...record },
        showEdit: true,
      });
    }
  };

  delete = record => {
    const { objId } = this.props;
    // 找出主键
    const { primaryKey } = this.state;
    const Keys = {};
    Keys[primaryKey] = record[primaryKey];
    deleteItem({ keys: JSON.stringify(Keys), objId }, () => {
      this.getPage();
    });
  };

  changeFormData = (value, key) => {
    this.setState(({ formData }) => ({ formData: { ...formData, [key]: value } }));
  };

  getPage = (params, values) => {
    const { objId } = this.props;
    const { formValues, pageData: pagination } = this.state;
    if (params == null) {
      params = {
        pageNo: pagination.current ? pagination.current : 1,
        pageSize: pagination.pageSize ? pagination.pageSize : 10,
        ...values,
      };
    }
    params = {
      ...params,
      query: JSON.stringify(formValues),
    };
    fetchData({ ...params, dataObjId: objId }, res => {
      if (!res.rows) {
        console.log('分页接口rows返回null报错');
      }
      this.setState({
        pageData: {
          list: res.rows,
          pagination: {
            current: res.pageNo,
            pageSize: res.pageSize,
            total: Number(res.total),
          },
        },
      });
    });
  };

  handleStandardTableChange = (pagination, filtersArg, sorter) => {
    const filters = Object.keys(filtersArg).reduce((obj, key) => {
      const newObj = { ...obj };
      newObj[key] = getValue(filtersArg[key]);
      return newObj;
    }, {});

    const params = {
      pageNo: pagination.current,
      pageSize: pagination.pageSize,
      ...filters,
    };
    if (sorter.field) {
      params.sorter = `${sorter.field}_${sorter.order}`;
    }
    this.getPage(params);
  };

  handleAdd = (fields, isAdd) => {
    const { formData } = this.state;
    const { objId } = this.props;
    add({ params: formData, objId, isAdd }, () => {
      message.success('操作成功');
      this.setState({
        modalVisible: false,
      });
      this.getPage();
    });
  };

  handleModalVisible = () => {
    this.setState({
      modalVisible: this.state.modalVisible ? false : true,
      formData: {},
    });
  };

  add = () => {
    const { formItem } = this.state;
    if (formItem.length < 10) {
      this.setState({
        modalVisible: true,
        formData: {},
        isAdd: true,
      });
    } else {
      this.setState({
        formData: {},
        showEdit: true,
        isAdd: true,
      });
    }
  };

  handleSelectRows = rows => {
    this.setState({
      selectedRows: rows,
    });
  };

  batchDelete = e => {
    const { selectedRows, formItem, objId } = this.state;

    if (!selectedRows) return;
    const Keys = formItem.filter(i => i.isPrimaryKey).reduce((acc, i) => {
      return { ...acc, [i.name]: selectedRows.map(row => row[i.name]).join(',') };
    });

    deleteItem({ keys: JSON.stringify(Keys), objId }, () => {
      this.setState({
        selectedRows: [],
      });
      this.getPage();
    });
  };

  goBack = () => {
    router.goBack();
  };

  formStateChange = (value, key) => {
    this.setState(({ formValues }) => ({ formValues: { ...formValues, [key]: value } }));
  };

  resetFormValues = () => {
    this.setState({
      formValues: {},
    });
  };

  render() {
    const {
      modalVisible,
      selectedRows,
      formItem,
      isAdd,
      objId,
      pageData,
      showEdit,
      formData,
      primaryKey,
      formValues,
      searchConfig,
    } = this.state;
    const { hasExport, hasImport } = this.props;

    const parentMethods = {
      handleAdd: this.handleAdd,
      handleModalVisible: this.handleModalVisible,
      formData,
      formItem,
      isAdd,
      changeFormData: this.changeFormData,
    };
    return (
      <>
        {showEdit ? (
          <Edit
            id={objId}
            recordId={formData[primaryKey]}
            recordKey={primaryKey}
            returnThis={() => {
              this.setState({
                showEdit: false,
              });
              this.getPage();
            }}
          />
        ) : (
          <Fragment>
            {searchConfig.length > 0 ? (
              <SearchDom
                formStateChange={this.formStateChange}
                formValues={formValues}
                getPage={this.getPage}
                resetFormValues={this.resetFormValues}
                config={{ condition: searchConfig, fromTab: true }}
              />
            ) : null}
            <Shell>
              <div style={{ paddingLeft: '10px', marginTop: '20px' }}>
                <ButtonDiy icon="plus" className="primaryBlue" handleClick={this.add} name="新建" />
             {/*   {hasImport && <ImportUtil objId={objId} />}
                {hasExport && <ExportInfo objId={objId} />}*/}
                <ButtonDiy handleClick={this.batchDelete} name="批量删除" />
              </div>
              <StandardTable
                rowKey="row_id"
                data={pageData}
                columns={this.columns}
                selectedRows={selectedRows}
                onSelectRow={this.handleSelectRows}
                onChange={this.handleStandardTableChange}
              />
            </Shell>
          </Fragment>
        )}

        <CreateForm {...parentMethods} modalVisible={modalVisible} />
      </>
    );
  }
}

export default FormList;
